import gzip


class GzipHandler:

    @staticmethod
    def origin2flow(request_data):
        _data = gzip.decompress(request_data)
        return _data

    @staticmethod
    def flow2origin(flow_data):
        _data = gzip.compress(flow_data)
        return _data



from lyrebird import application
from lyrebird.log import get_logger
from pathlib import Path
from copy import deepcopy
import imp
import traceback
import datetime


logger = get_logger()


class Reporter:

    def __init__(self):
        self.scripts = []
        workspace = application.config.get('reporter.workspace')
        if not workspace:
            logger.debug(f'reporter.workspace not set.')
        else:
            self._read_reporter(workspace)
            logger.debug(f'Load statistics scripts {self.scripts}')

    def _read_reporter(self, workspace):
        target_dir = Path(workspace)
        if not target_dir.exists():
            logger.error('Reporter workspace not found')
        for report_script_file in target_dir.iterdir():
            if report_script_file.name.startswith('_'):
                continue
            if not report_script_file.is_file():
                logger.warning(f'Skip report script: is not a file, {report_script_file}')
                continue
            if report_script_file.suffix != '.py':
                logger.warning(f'Skip report script: is not a python file, {report_script_file}')
                continue
            try:
                _script_module = imp.load_source('reporter_script', str(report_script_file))
            except Exception:
                logger.warning(
                    f'Skip report script: load script failed, {report_script_file}\n{traceback.format_exc()}')
                continue
            if not hasattr(_script_module, 'report'):
                logger.warning(f'Skip report script: not found a report method in script, {report_script_file}')
                continue
            if not callable(_script_module.report):
                logger.warning(f'Skip report script: report method not callable, {report_script_file}')
                continue
            self.scripts.append(_script_module.report)

    def report(self, data):
        task_manager = application.server.get('task')

        def send_report():
            new_data = deepcopy(data)
            for script in self.scripts:
                try:
                    script(new_data)
                except Exception:
                    logger.error(f'Send report failed:\n{traceback.format_exc()}')
        task_manager.add_task('send-report', send_report)


last_page = None
last_page_in_time = None
lyrebird_start_time = None


def _page_out():
    global last_page
    global last_page_in_time

    if last_page and last_page_in_time:
        duration = datetime.datetime.now() - last_page_in_time
        application.server['event'].publish('page.out', {
            'page': last_page,
            'duration': duration.total_seconds()
        })

        # TODO remove below
        application.reporter.report({
            'action': 'page.out',
            'page': last_page,
            'duration': duration.total_seconds()
        })


def page_in(name):
    _page_out()

    global last_page
    global last_page_in_time

    application.server['event'].publish('page.in', {
        'page': name
    })

    # TODO remove below
    application.reporter.report({
        'action': 'page.in',
        'page': name
    })

    last_page = name
    last_page_in_time = datetime.datetime.now()


def start():
    global lyrebird_start_time
    lyrebird_start_time = datetime.datetime.now()
    application.server['event'].publish('start', {})

    # TODO remove below
    application.reporter.report({
        'action': 'start'
    })


def stop():
    _page_out()
    application.server['event'].publish('stop', {
        'duration': (datetime.datetime.now() - lyrebird_start_time).total_seconds()
    })

    # TODO remove below
    application.reporter.report({
        'action': 'stop',
        'duration': (datetime.datetime.now() - lyrebird_start_time).total_seconds()
    })



from flask_restful import Resource
from lyrebird.mock import context
from flask import request
import traceback
from lyrebird import application, log

logger = log.get_logger()


class MockGroup(Resource):
    """
    mock数据组
    """

    def get(self, group_id=None, label=None):
        if group_id:
            _group = context.application.data_manager.id_map.get(group_id)
            if not _group:
                return context.make_fail_response('Not found group')

            return application.make_ok_response(data=_group)

        context.application.data_manager.reload()
        root = context.application.data_manager.root
        if not label:
            return application.make_ok_response(data=root)

        # update mock data tree with label
        groups_set = set()

        label_list = label.split('+')
        for label_id in label_list:
            label = application.labels.label_map.get(label_id)
            if not label:
                return application.make_fail_response(f'Label {label_id} is not found!')

            if not groups_set:
                groups_set = set(label['groups'])
            else:
                groups_set = groups_set & set(label['groups'])

        data_map = context.application.data_manager.make_data_map_by_group(groups_set)
        return application.make_ok_response(data=data_map)


    def post(self):
        data = request.json.get('data')
        if not data:
            data = {
                'name': request.json.get('name'),
                'parent_id': request.json.get('parent_id')
            }
        group_id = context.application.data_manager.add_group(data)
        return context.make_ok_response(data={'group_id': group_id})

    def put(self):
        group_id = request.json.get('id')
        data = request.json.get('data')
        context.application.data_manager.update_group(group_id, data)
        return context.make_ok_response()

    def delete(self, group_id=None):
        if group_id is not None:
            context.application.data_manager.delete(group_id)
            return context.make_ok_response()

        query = request.json.get('query')
        if query is None:
            return application.make_fail_response(f'Delete query is None!')

        if context.application.data_manager.is_deleting_lock:
            return application.make_fail_response(f'Is deleting, no new delete')

        context.application.data_manager.is_deleting_lock = True
        try:
            context.application.data_manager.delete_by_query(query)
            # 是否需要前端可以控制是否刷新？
        except Exception as e:
            logger.error(f'Delete error: {traceback.format_exc()}')
            return application.make_fail_response(f'Delete failure: {str(e)}')
        finally:
            context.application.data_manager.is_deleting_lock = False

        context.application.data_manager.reactive()
        return context.make_ok_response()


class MockData(Resource):
    """
    mock数据
    """

    def get(self, _id):
        data = context.application.data_manager.get(_id)
        display_item = {}
        # Import decoder for decoding the requested content
        application.encoders_decoders.decoder_handler(data, output=display_item)
        return application.make_ok_response(data=display_item)

    def put(self):
        data_id = request.json.get('id')
        data = request.json
        # Import encoder for encoding the requested content
        application.encoders_decoders.encoder_handler(data)
        context.application.data_manager.update_data(data_id, data)
        context.application.data_manager.reactive()
        return context.make_ok_response()

    def post(self):
        parent_id = request.json.get('parent_id')
        data = request.json.get('data')
        new_data_id = context.application.data_manager.add_data(parent_id, data)
        return context.make_ok_response(data_id=new_data_id)

    def delete(self, _id):
        context.application.data_manager.delete(_id)
        return context.make_ok_response()


class ActivatedMockGroup(Resource):

    def get(self):
        return context.make_ok_response(
            data=context.application.data_manager.activated_group
        )

    def put(self, group_id=None, action=None):
        if action == 'activate':
            # Only one group could be activated
            context.application.data_manager.activate(group_id)
        elif action == 'deactivate':
            context.application.data_manager.deactivate()
        else:
            logger.warning('DeprecationWarning: Deactivate with no action parameter is deprecated soon,\
                use /mock/<string:group_id>/deactivate instead')
            context.application.data_manager.deactivate()
        return context.make_ok_response()


class MockGroupByName(Resource):

    def put(self):
        group_name = request.json.get('group_name')
        action = request.json.get('action', 'activate')
        for _id in context.application.data_manager.id_map:
            group = context.application.data_manager.id_map.get(_id)
            if group['name'] == group_name:
                if action == 'activate':
                    context.application.data_manager.activate(group['id'])
                elif action == 'deactivate':
                    context.application.data_manager.deactivate()
                else:
                    logger.warning('DeprecationWarning: Deactivate with no action parameter is deprecated soon,\
                        use /mock/<string:group_id>/deactivate instead')
                    context.application.data_manager.deactivate()
                return context.make_ok_response()
        return context.make_fail_response(f'Group not found. name={group_name}')


class MockDataLabel(Resource):
    """
    mock数据标签
    """

    def get(self):
        application.labels.get_label(context.application.data_manager.root)
        return application.make_ok_response(labels=application.labels.label_map)

    def post(self):
        label = request.json.get('label')
        required_key = ['name']
        missed_required_key = [key for key in required_key if not label.get(key)]
        if missed_required_key:
            return application.make_fail_response(f'Label {" ".join(missed_required_key)} is required!')

        label_id = application.labels._get_label_name_md5(label)
        if label_id in application.labels.label_map:
            return application.make_fail_response(f'Label {label["name"]} existed!')

        application.labels.create_label(label)
        return context.make_ok_response()

    def put(self):
        label = request.json.get('label')
        label_id = label.get('id')
        if not label_id:
            return application.make_fail_response('Label id is required!')
        if not label_id in application.labels.label_map:
            return application.make_fail_response(f'Label {label.get("name")} not found!')

        application.labels.update_label(label)
        return context.make_ok_response()

    def delete(self):
        label_id = request.json.get('id')
        if not label_id:
            return application.make_fail_response('Label id is required!')
        if not label_id in application.labels.label_map:
            return application.make_fail_response(f'Label {label_id} not found!')

        application.labels.delete_label(label_id)
        return context.make_ok_response()



from lyrebird import on_request
from lyrebird.checker import ExtensionCategory

TITLE = '<示例脚本>在Request中添加Param'
CATEGORY = ExtensionCategory.MODIFIER

@on_request(rules={
    "request.url": "(?=.*poi/detail)"
})
def add_request_param(flow):
    if '?' in flow['request']['url']:
        flow['request']['url'] += '&param=1'
    else:
        flow['request']['url'] += '?param=1'





from lyrebird.plugins import manifest
from . import handler
from . import status


manifest(
    id='demo',
    name='Demo',
    # Pick an icon from https://webmaterialdesignicons.com/
    # Example icon='mdi-video-outline',
    icon='',
    api=[
        # http://localhost:9090/plugins/demo/api/list
        ('/api/list', handler.request_list, ['GET']),
        # http://localhost:9090/plugins/demo/api/mock
        ('/api/mock', handler.mock, ['POST']),
        # http://localhost:9090/plugins/demo/api/reset
        ('/api/reset', handler.reset, ['PUT'])
    ],
    background=[
    ],
    event=[
        ('flow', handler.on_request)
    ],
    status=[
        status.StatusbarDemoImage,
        status.StatusbarDemoText
    ]
)



import urllib
import requests
from requests.packages import urllib3
from flask import Response, jsonify, stream_with_context
from .. import context
from lyrebird import application
from lyrebird.log import get_logger
from lyrebird.mock import lb_http_status
from .duplicate_request_handler import DuplicateRequest
import traceback


# 关闭ssl警告
urllib3.disable_warnings()

logger = get_logger()


class ProxyHandler:
    """
    按照代理规则代理

    """

    def handle(self, handler_context):

        request = handler_context.flow['request']

        origin_url = request.get('url')

        logger.info(f'<Proxy> {origin_url}')

        if not origin_url:
            handler_context.is_proxiable = False
            return

        parsed_url = urllib.parse.urlparse(origin_url)
        if not parsed_url.hostname:
            handler_context.is_proxiable = False
            return
        elif parsed_url.hostname in ['localhost', '127.0.0.1', ] and parsed_url.port == application.config["mock.port"]:
            DuplicateRequest().handle(handler_context)
            return

        data = handler_context.get_request_body()

        method = request['method']
        headers = handler_context.get_request_headers()

        try:
            r = requests.request(
                method, 
                origin_url, 
                headers=headers, 
                data=data, 
                cookies=handler_context.request.cookies, 
                stream=True, 
                verify=False, 
                allow_redirects=False)
            logger.info(f'<Proxy> SUCCESS {r.status_code} {origin_url}')
            
        except:
            trace_str = traceback.format_exc()
            error_response = {
                'code': 3000,
                'message': 'proxy error',
                'trace': trace_str
            } 
            resp = jsonify(error_response)
            resp.status = lb_http_status.STATUS_PROXY_ERROR
            handler_context.response = resp
            logger.info(f'<Proxy> PROXY ERROR {origin_url}\n------\ntrace:\n{trace_str}\n------\n<Proxy> PROXY ERROR {origin_url}')
            return

        # 增加数据源标记，此数据经代理得到
        resp_headers = [('lyrebird', 'proxy')]
        for name, value in r.raw.headers.items():
            # rm 'content-length' from ignore list
            if name.lower() in ('content-encoding',
                                'transfer-encoding'):
                continue
            if name.lower() == 'content-length' and 'content-encoding' in r.headers and r.headers['content-encoding'] == 'gzip':
                # 如果是gzip请求，由于requests自动解压gzip，所以此处抹去content-length,以匹配解压后的数据长度
                continue
            resp_headers.append((name, value))
        
        # HTTP Status code 204 => No content
        if r.status_code == 204:
            handler_context.response = Response(None, status=r.status_code, headers=resp_headers)
            return

        # After huangyuanzhen test, we use 2048byte buffer :D
        handler_context.response = Response(
            stream_with_context(r.iter_content(chunk_size=handler_context.response_chunk_size)),
            status=r.status_code,
            headers=resp_headers)



from flask_restful import Resource
from flask import request
from lyrebird.mock import context
from lyrebird import application, reporter


class Menu(Resource):

    def get(self):
        menu = [
            {
                'name': 'inspector',
                'title': 'Inspector',
                'type': 'router',
                'path': '/',
                'icon': 'mdi-monitor'
            },
            {
                'name': 'datamanager',
                'title': 'DataManager',
                'type': 'router',
                'path': '/datamanager',
                'icon': 'mdi-database-plus'
            },
            {
                'name': 'checker',
                'title': 'Extension',
                'type': 'router',
                'path': '/checker',
                'icon': 'mdi-script-text-outline'
            },
            {
                'name': 'inspector-pro',
                'title': 'Inspector Pro',
                'type': 'router',
                'path': '/inspector-pro',
                'icon': 'mdi-timeline-outline'
            }]

        # Load plugins from new plugin manager
        _pm = application.server['plugin']
        for plugin_id, plugin in _pm.plugins.items():
            menu.append({
                'name': 'plugin-container',
                'title': plugin.manifest.name,
                'type': 'router',
                'path': '/plugins',
                'icon': plugin.manifest.icon,
                'params': {
                    'src': f'/plugins/{plugin_id}',
                    'name': plugin_id
                }
            })
        # When there is no actived menu, the first one is displayed by default
        if not application.active_menu:
            self.set_active_menu(menu[0])
        active_menu = application.active_menu
        for index, menu_item in enumerate(menu):
            if menu_item['name'] == active_menu.get('name'):
                active_menu_index = index
                break
        return context.make_ok_response(menu=menu, activeMenuItem=active_menu, activeMenuItemIndex=active_menu_index)

    def put(self):
        active_menu = request.json.get('activeMenuItem')
        self.set_active_menu(active_menu)
        return context.make_ok_response()

    def set_active_menu(self, menu):
        application.active_menu = menu
        if menu.get('params', {}).get('name'):
            menu_name = menu['params']['name']
        else:
            menu_name = menu['name']
        reporter.page_in(menu_name)



import re
import os
import math
import time
import socket
import tarfile
import requests
import datetime
import netifaces
from pathlib import Path
from contextlib import closing
from lyrebird.log import get_logger

logger = get_logger()


def convert_size(size_bytes):
    if size_bytes == 0:
        return "0B"
    size_name = ("B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB")
    i = int(math.floor(math.log(size_bytes, 1024)))
    p = math.pow(1024, i)
    s = round(size_bytes / p, 2)
    return "%s %s" % (s, size_name[i])


def convert_time(duration):
    if duration < 1:
        return str(round(duration * 1000)) + 'ms'
    else:
        return str(round(duration, 2)) + 's'


def timeit(method):
    def timed(*args, **kw):
        ts = time.time()
        result = method(*args, **kw)
        te = time.time()
        print(f'{method.__name__} execution time {(te-ts)*1000}')
        return result
    return timed


def is_port_in_use(port, host='127.0.0.1'):
    sock = None
    try:
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.settimeout(1)
        sock.connect((host, int(port)))
        return True
    except socket.error:
        return False
    finally:
        if sock:
            sock.close()


def find_free_port():
    with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as s:
        s.bind(('', 0))
        s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        return s.getsockname()[1]


def get_ip():
    """
    Get local ip from socket connection

    :return: IP Addr string
    """
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.connect(('8.8.8.8', 80))
    return s.getsockname()[0]


def get_interface_ipv4():
    ipv4_list = []
    interface_list = netifaces.interfaces()
    for interface_name in interface_list:
        if interface_name == 'lo0':
            continue
        interface_dict = netifaces.ifaddresses(interface_name)
        interface = interface_dict.get(netifaces.AF_INET)
        if not interface:
            continue
        for alias in interface:
            ipv4_list.append({
                'address': alias['addr'],
                'netmask': alias['netmask'],
                'interface': interface_name
            })
    return ipv4_list


def compress_tar(input_path, output_path, suffix=None):
    current_path = Path.cwd()
    input_path = Path(input_path).expanduser().absolute().resolve()
    output_path = Path(output_path).expanduser().absolute().resolve()
    output_file = f'{output_path}{suffix}' if suffix else output_path

    os.chdir(input_path)
    # If not chdir, the directory in the compressed file will start from the root directory
    tar = tarfile.open(output_file, 'w:gz')
    for root, dirs, files in os.walk(input_path):
        for f in files:
            tar.add(f, recursive=False)
    tar.close()
    os.chdir(current_path)
    return output_file


def decompress_tar(input_path, output_path=None):
    input_path = Path(input_path).expanduser().absolute().resolve()
    if not output_path:
        filename = input_path.stem if input_path.suffix else f'{input_path.name}-decompress'
        output_path = input_path.parent / filename

    tf = tarfile.open(str(input_path))
    tf.extractall(str(output_path))
    tf.close()
    return output_path


def download(link, input_path):
    resp = requests.get(link, stream=True)
    with open(input_path, 'wb') as f:
        for chunck in resp.iter_content():
            f.write(chunck)


class CaseInsensitiveDict(dict):
    '''
    A dict data-structure that ignore key's case.
    Any read or write related operations will igonre key's case.

    Example: 
    <key: 'abc'> & <key: 'ABC'> will be treated as the same key, only one exists in this dict.
    '''
    
    def __init__(self, raw_dict):
        self.__key_map = {}
        for k, v in raw_dict.items():
            self.__setitem__(k, v)

    def __get_real_key(self, key):
        return self.__key_map.get(key.lower(), key)

    def __set_real_key(self, real_key):
        if real_key.lower() not in self.__key_map:
            self.__key_map[real_key.lower()] = real_key

    def __pop_real_key(self, key):
        return self.__key_map.pop(key.lower())

    def __del_real_key(self, key):
        del self.__key_map[key.lower()]

    def __clear_key_map(self):
        self.__key_map.clear()

    def __setitem__(self, key, value):
        real_key = self.__get_real_key(key)
        self.__set_real_key(real_key)
        super(CaseInsensitiveDict, self).__setitem__(real_key, value)

    def __getitem__(self, key):
        return super(CaseInsensitiveDict, self).__getitem__(self.__get_real_key(key))

    def get(self, key, default=None):
        return super(CaseInsensitiveDict, self).get(self.__get_real_key(key), default)

    def __delitem__(self, key):
        real_key = self.__pop_real_key(key)
        return super(CaseInsensitiveDict, self).__delitem__(real_key)

    def __contains__(self, key):
        return key.lower() in self.__key_map

    def pop(self, key):
        real_key = self.__pop_real_key(key)
        return super(CaseInsensitiveDict, self).pop(real_key)

    def popitem(self):
        item = super(CaseInsensitiveDict, self).popitem()
        if item:
            self.__del_real_key(item[0])
        return item

    def clear(self):
        self.__clear_key_map()
        return super(CaseInsensitiveDict, self).clear()

    def update(self, __m=None, **kwargs) -> None:
        if __m:
            for k, v in __m.items():
                self.__setitem__(k, v)
        if kwargs:
            for k, v in kwargs.items():
                self.__setitem__(k, v)

class HookedDict(dict):
    '''
    Hook build-in dict to protect CaseInsensitiveDict data type.
    Only <headers> value is CaseInsensitiveDict at present.
    '''
    
    def __init__(self, raw_dict):
        for k, v in raw_dict.items():
            if type(v) == dict:
                if k.lower() == 'headers':
                    v = CaseInsensitiveDict(v)
                else:
                    v = HookedDict(v)
            self.__setitem__(k, v)

    def __setitem__(self, __k, __v) -> None:
        if type(__v) == dict:
            if __k.lower() == 'headers':
                __v = CaseInsensitiveDict(__v)
            else:
                __v = HookedDict(__v)
        return super(HookedDict, self).__setitem__(__k, __v)

class TargetMatch:

    @staticmethod
    def is_match(target, pattern):
        if not TargetMatch._match_type(target, pattern):
            return False
        if type(target) == str:
            return TargetMatch._match_string(target, pattern)
        elif type(target) in [int, float]:
            return TargetMatch._match_numbers(target, pattern)
        elif type(target) == bool:
            return TargetMatch._match_boolean(target, pattern)
        elif type(target).__name__ == 'NoneType':
            return TargetMatch._match_null(target, pattern)
        else:
            logger.warning(f'Illegal match target type: {type(target)}')
            return False

    @staticmethod
    def _match_type(target, pattern):
        return isinstance(target, type(pattern))

    @staticmethod
    def _match_string(target, pattern):
        return True if re.search(pattern, target) else False

    @staticmethod
    def _match_numbers(target, pattern):
        return target == pattern

    @staticmethod
    def _match_boolean(target, pattern):
        return target == pattern

    @staticmethod
    def _match_null(target, pattern):
        return target == pattern


class JSONFormat:

    def json(self):
        prop_collection = {}
        props = dir(self)
        for prop in props:
            if prop.startswith('_'):
                continue
            prop_obj = getattr(self, prop)
            if isinstance(prop_obj, (str, int, bool, float)):
                prop_collection[prop] = prop_obj
            elif isinstance(prop_obj, datetime.datetime):
                prop_collection[prop] = prop_obj.timestamp()
        return prop_collection



import runpy
import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))

VERSION = runpy.run_path(
    os.path.join(here, "lyrebird", "version.py")
)["VERSION"]

with open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

with open(os.path.join(here, 'install_requires.txt'), encoding='utf-8') as f:
    install_requires_str = f.read()
    install_requires = install_requires_str.split()

setup(
    name='lyrebird',
    version=VERSION,
    packages=['lyrebird'],
    url='https://github.com/meituan/lyrebird',
    author='HBQA',
    long_description=long_description,
    long_description_content_type="text/markdown",
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS",
        "Operating System :: Microsoft :: Windows",
    ],
    entry_points={
        'console_scripts': [
            'lyrebird = lyrebird.manager:main'
        ]
    },
    install_requires=install_requires,
    extras_require={
        'dev': [
            "autopep8",
            "pylint",
            "pytest",
            "pytest-cov"
        ]
    }
)



from flask import Flask, redirect, url_for
from . import context
from .blueprints.apis import api
from .blueprints.ui import ui
from .blueprints.core import core
from flask_socketio import SocketIO
from ..version import VERSION
from lyrebird.base_server import ThreadServer
from lyrebird import application
from lyrebird import log
import traceback

"""
Mock server

Lyrebird main server
Default port : 9090

* HTTP mock
* HTTP record
* Lyrebird UI
* Lyrebird API
* Lyrebird plugin management
"""

_logger = log.get_logger()


class LyrebirdMockServer(ThreadServer):
    """
    模拟接口服务
    使用flask在默认的9090端口启动，模拟线上接口，同时支持通过api动态修改接口数据。

    """

    def __init__(self):
        super().__init__()

        self.conf = application.config
        # TODO rm conf rom mock context
        context.application.conf = application.config

        self.debug = False
        self.port = 9090
        self._working_thread = None
        self.app = Flask('MOCK')

        self.app.env = 'development'

        # async_mode = threading / eventlet / gevent / gevent_uwsgi
        self.socket_io = SocketIO(self.app, async_mode='threading', logger=False, cors_allowed_origins='*')

        # 存储socket-io
        context.application.socket_io = self.socket_io
        application._socketio = self.socket_io

        # 生成过滤器实例
        if self.conf:
            self.port = self.conf.get('mock.port')
        else:
            _logger.error('Can not start mock server without config file')
            raise SyntaxError('Can not start mock server without config file.'
                              ' Default config file path = api-mock/conf.json')

        # Register blueprints
        self.app.register_blueprint(api)
        self.app.register_blueprint(core)
        self.app.register_blueprint(ui)

        @self.app.route('/')
        def index():
            """
            设置默认页面为UI首页
            """
            return redirect(url_for('ui.index')+f'?v={VERSION}')

        @self.app.errorhandler(Exception)
        def on_app_error(error):
            _logger.error(f'[mocker server exception]: {error.__class__.__name__} {error}\n{traceback.format_exc()}')
            return application.make_fail_response(f'{error.__class__.__name__} {error}\n{traceback.format_exc()}')

    def run(self):
        server_ip = application.config.get('ip')
        _logger.warning(f'Core start on http://{server_ip}:{self.port}')
        self.socket_io.run(self.app, host='0.0.0.0', port=self.port, debug=self.debug, use_reloader=False)

    def stop(self):
        """
        停止服务

        """
        super().stop()
        try:
            self.socket_io.stop()
        except Exception:
            pass
        _logger.warning('CoreServer shutdown')



from lyrebird.mock import headers

def kill_flow(flow):
    request_headers = flow['request']['headers']
    if not request_headers.get(headers.MITMPROXY_CLIENT_ADDRESS):
        return

    flow['response']['code'] = 0
    flow['response']['headers'] = {}
    flow['response']['data'] = ''

    flow['response']['headers'][headers.MITMPROXY_COMMAND] = 'kill'
